/** \file maximDeviceSpecificUtilities.c ****************************************
 * 
 *             Project: Maxim Plug-in Peripheral Modules
 *            Filename: maximDeviceSpecificUtilities.c
 *         Description: This module contains the functions which are specific
 *                      to the invidiual modules.  These low level functions 
 *                      are not specific to the example/demo programs and could
 *                      be cut/pasted into the user's application as a starting
 *                      point for development of an end application.
 *
 *    Revision History:
 *\n       7-20-12 Rev 1.4 Nathan Young    Initial release
 * 
 *  --------------------------------------------------------------------
 * 
 *    This code follows the following naming conventions.
 * 
 * 	  char                   chPmodValue
 * 	  char (array)           sPmodString[16]
 * 	  float                  fPmodValue
 * 	  int                    nPmodValue
 * 	  int (array)            anPmodValue[16]
 * 	  u16                    uPmodValue
 * 	  u8                     uchPmodValue
 * 	  u8 (array)             auchPmodBuffer[16]
 * 	  unsigned int           unPmodValue
 * 	  int *                  punPmodValue
 * 
 * ------------------------------------------------------------------------- */
/*
 * Copyright (C) 2012 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products retains all ownership rights.
 *
 ***************************************************************************/

#include "maximPMOD.h"
#include "maximDeviceSpecificUtilities.h"
#include "menu.h"


int max_MAX44000_set_prox_mode(u32 unPeripheralAddressI2C)
/**
* \brief       Activate the proximity mode on the MAX44000.
* \par         Details
*              This sets up the main configuration register (reg addr 0x00) as follows:
* \n           MODE[2:0] = 101 : "PROX only continuously"
* \n           TRIM = 1 : factory programmed gains for green and IR channels
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in MicroBlaze memory map
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal = TRUE;
	u8 auchTxBuffer[2];
	int nByteCount=0;

	auchTxBuffer[0] = 0x01;  // Main configuration register
	auchTxBuffer[1] = 0x34;  // Prox mode with (default) factory trimmed gains
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
								(u8 *)&auchTxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;
	return(nReturnVal);
}

int max_MAX44000_set_als_mode(u32 unPeripheralAddressI2C)
/**
* \brief       Activate the ambient light sensing mode on the MAX44000.
* \par         Details
*              This sets up the main configuration register (reg addr 0x00) as follows:
* \n           MODE[2:0] = 001 : "ALS-G-IR". (difference between green and IR.)
* \n           TRIM = 1 : factory programmed gains for green and IR channels
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal = TRUE;
	u8 auchTxBuffer[2];
	int nByteCount=0;

	auchTxBuffer[0] = 0x01;  // Main configuration register
	auchTxBuffer[1] = 0x24;  // Ambient light sensor mode with (default) factory trimmed gains
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
								(u8 *)&auchTxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;
	return(nReturnVal);
}

int max_MAX44000_set_led_drive_current(u32 unPeripheralAddressI2C, u8 uchLedDriveCurrent)
/**
* \brief       Set the LED drive current for the IRDA LED external to the IC.  
* \par         Details
*              Recommended values for this current are between 10-30mA (corresponding to a setting of 1-3).
*              This sets up the transmit configuration register (reg addr 0x03) as follows:
* \n           DRV [3:0] = 001 : "ALS-G-IR". (difference between green and IR.)
*
* \param[in]   unPeripheralAddressI2C   - address of the I2C peripheral in Microblaze memory map
* \param[in]   uchLedDriveCurrent       - LED drive value = (LED current/10)
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal = TRUE;
	u8 auchTxBuffer[2];
	int nByteCount=0;

	auchTxBuffer[0] = 0x03;  // LED Drive Current register
	auchTxBuffer[1] = uchLedDriveCurrent;  // recommended values are 1-3
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
						(u8 *)&auchTxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	return(nReturnVal);
}

// This function reads and returns the Proximity detector value on the MAX44000
int max_MAX44000_readProx(u32 unPeripheralAddressI2C, unsigned int *unProxReading)
/**
* \brief       Read the proximity detector value from the MAX44000.  
* \par         Details
*              Recommended values for this current are between 10-30mA (corresponding to a setting of 1-3).
*              This sets up the transmit configuration register (reg addr 0x03) as follows:
* \n           DRV [3:0] = 001 : "ALS-G-IR". (difference between green and IR.)
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map
* \param[out]  *unProxReading            - proximity detector value is stored at unProxReading
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal = TRUE;
	u8 auchTxBuffer[2];
	u8 auchRxBuffer[2];
	unsigned int unLowerByte=0;
	int nByteCount=0;

	auchTxBuffer[0] = 0x16;  // Proximity Sensor Data register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
			(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
			(u8 *)&auchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	unLowerByte = auchRxBuffer[0];
	if(nReturnVal==TRUE)
		*unProxReading = unLowerByte;

		delay(ABOUT_ONE_SECOND / 10);

	return(nReturnVal);
}


int max_MAX44000_readLux(u32 unPeripheralAddressI2C, unsigned int *unLuxReading)
/**
* \brief       Return the Lux value from the MAX44000.  
* \par         Details
*              This function implements a state machine to manage the transaction.  The state 
*              machine handles error cases, including bit overflow, changing the gain setting
*              and then re-reading the lux value.  See source for additional comments/theory.
*              This sets up the transmit configuration register (reg addr 0x03) as follows:
* \n           DRV [3:0] = 001 : "ALS-G-IR". (difference between green and IR.)
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map
* \param[out]  *unLuxReading             - lux reading is stored at unLuxReading
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal = TRUE;
	u8 auchTxBuffer[2];
	u8 auchRxBuffer[2];
	int nByteCount=0;
	unsigned int unTempLux=0;
	u8 uchStateMachineActive=TRUE;
	int nState=0;
	u8 uchALSPGA=0;
	u8 uchHighByte=0;
	u8 uchLowByte=0;
	int nNumberFailedReadWrites=0;

	// Device Theory

	// The ambient light sensor reading is retrieved by reading a 14 bit unsigned integer value held in
	//   bits [5:0] of register 0x04 and [7:0] of 0x05.
	// To convert the ALSDATA[13:0] to lux, it is scaled by (1/32, 1/8, 1/2, or 4) depending on the value of ALSPGA
	// ALSPGA[1:0] = register 0x02, bits [1:0]
	// Additionally, the light sensor can overflow (bit 6 of register 0x04), in which case the reading is invalid
	//   and the ALSPGA setting needs to be increased

	// This function attempts to read register 0x04, then checks for the overflow bit.
	// If overflow is set, it increases ALSPGA, delays, re-reads 0x04.
	// If overflow remains set, it returns FALSE.

	// In 14 bit mode, the MAX44000 device requires 100mS to refresh its internal registers.
	// Since this function uses polling mode (instead of interrupt mode), to be conservative,
	//   all delays are chosen to be ~250mS.

	while(uchStateMachineActive==TRUE)
	{
		switch(nState)
		{
			case MAX44000_STATE_BEGIN:
				nState = MAX44000_STATE_SET_ALSPGA;
				break;
			case MAX44000_STATE_SET_ALSPGA:
				auchTxBuffer[0] = 0x02;  // Receive configuration register
				auchTxBuffer[1] = 0x00;  
				uchALSPGA = 0x00;		 // Sets up 14-bit resolution, gain = 0.03125 LUX/lsb
				nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
					(u8 *)&auchTxBuffer, 2, XIIC_STOP);

				if(nByteCount==2)
					nState = MAX44000_STATE_READ_04;
				else
					nState = MAX44000_STATE_FAILED_RW;

				delay(ABOUT_ONE_SECOND/4); // 250mS
				break;
			case MAX44000_STATE_READ_04:
				auchTxBuffer[0] = 0x04;  // Lux High-Byte register (overflow bit + bits 13:8 of lux reading)
				nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
						(u8 *)&auchTxBuffer, 1, XIIC_STOP);

				if(nByteCount==1)
				{
					nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
							(u8 *)&auchRxBuffer, 1, XIIC_STOP);
					if(nByteCount==1)
					{
						// Check for overflow bit first
						if((auchRxBuffer[0] & 0x40)==0x40)
							nState = MAX44000_STATE_INCREASE_ALSPGA;
						else
						{
							uchHighByte=auchRxBuffer[0];
							nState = MAX44000_STATE_READ_05;
						}
					}
					else
						nState = MAX44000_STATE_FAILED_RW;
				}
				else
					nState = MAX44000_STATE_FAILED_RW;

				break;
			case MAX44000_STATE_READ_05:
				auchTxBuffer[0] = 0x05;  // Lux Low-Byte register (bits 7:0 of lux reading)
				nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
						(u8 *)&auchTxBuffer, 1, XIIC_STOP);

				if(nByteCount==1)
				{
					nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
							(u8 *)&auchRxBuffer, 1, XIIC_STOP);
					if(nByteCount==1)
					{
						uchLowByte=auchRxBuffer[0];
						nState = MAX44000_STATE_PROCESS_LUX;
					}
					else
						nState = MAX44000_STATE_FAILED_RW;
				}
				else
					nState = MAX44000_STATE_FAILED_RW;
				break;
			case MAX44000_STATE_INCREASE_ALSPGA:
				uchALSPGA++;
				if(uchALSPGA >=4)
				{
					unTempLux = 16383;  // (maximum value for a 14 bit unsigned int number)
					nState = MAX44000_STATE_EXIT_SUCCESS;
				}
				else
				{
					auchTxBuffer[0] = 0x02;  // Recv config register (ADC conv time and PGA gain setting
					auchTxBuffer[1] = (uchALSPGA & 0x03);  
					nByteCount = XIic_Send(unPeripheralAddressI2C, MAX44000_IIC_ADDRESS,
							(u8 *)&auchTxBuffer, 2, XIIC_STOP);

					if(nByteCount==2)
						nState = MAX44000_STATE_READ_04;
					else
						nState = MAX44000_STATE_FAILED_RW;

					// 100mS per reading, so we'll delay about twice that amount before progressing
					delay(ABOUT_ONE_SECOND/4);
				}
				break;
			case MAX44000_STATE_PROCESS_LUX:
				unTempLux=0;
				unTempLux = (uchHighByte & 0x3F);
				unTempLux = unTempLux << 8;
				unTempLux += uchLowByte;

				nState =MAX44000_STATE_EXIT_SUCCESS;
				if(uchALSPGA==0)
					unTempLux /= 32;
				else if(uchALSPGA==1)
					unTempLux /= 8;
				else if(uchALSPGA==2)
					unTempLux /= 2;
				else if(uchALSPGA==3)
					unTempLux *= 4;
				else
				{
					nState = MAX44000_STATE_EXIT_FAILED;
				}

				break;
			case MAX44000_STATE_FAILED_RW:
				nNumberFailedReadWrites++;
				printf("Error.  Failed to read or write I2C device:\r\n");
				if(nNumberFailedReadWrites>10)  // arbitrary 10 attempts
					nState = MAX44000_STATE_EXIT_FAILED;
				else
					nState = MAX44000_STATE_DELAY;
				break;
			case MAX44000_STATE_DELAY:
				delay(ABOUT_ONE_SECOND / 4);
				nState = MAX44000_STATE_BEGIN;
				break;
			case MAX44000_STATE_EXIT_FAILED:
				uchStateMachineActive=FALSE;
				nReturnVal=FALSE;
				break;
			case MAX44000_STATE_EXIT_SUCCESS:
				uchStateMachineActive=FALSE;
				*unLuxReading = unTempLux;
				nReturnVal=TRUE;
				break;
		}
	}

	return(nReturnVal);
}

int max_MAX9611_read_silicon_die_temp(u32 unPeripheralAddressI2C, int *pnSiliconDieTemperature)
/**
* \brief       Read an 8-bit, two's compliment value of the temperature sensor on the MAX9611 die.
* \par         Details
*              Combines 8MSBs from reg addr 0x08 with 1 LSB from reg addr 0x09 to form a 9-bit, two's  compliment
*              result.  That result represents .48 degreeC per bit which is then divided by two to return an 8-bit,
*              two's compliment value representing 0.96 degreeC per bit.  For higher accuracy, the last few 
*              instructions of this function could be modified using floating point operations.
*
* \param[in]   unPeripheralAddressI2C      - address of the I2C peripheral in Microblaze memory map
* \param[out]  *pnSiliconDieTemperature    - temperature reading is stored at pnSiliconDieTemperature
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer=0;
	u8 uchRxBuffer=0;
	int nByteCount=0;
	int nTemperature=0;
	int nReturnVal=TRUE;

	uchTxBuffer = 0x08;  // MSB 8 bits of temp register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
			nReturnVal=FALSE;

	// The 8 bits from the MSB register represent bits[8:1] of the 9 bit temperature reading
	// shift this value 1 bit left to move [7:0] to [8:1]
	nTemperature = (int)uchRxBuffer;
	nTemperature = nTemperature << 1;

	uchTxBuffer = 0x09;  // LSB 8 bits of temp register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
			nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
			nReturnVal=FALSE;

	// Now, assign this value to the
	if(nReturnVal==TRUE)
	{
		// The upper bit of uchRxBuffer contains the LSB of the 9 bit temp
		nTemperature = nTemperature + ((uchRxBuffer & 0x0080)>> 7);

		// Now, check if the 9th bit is set, meaning it is a negative number
		// If so, sign extend the number (since it is 2s complement)
		if((nTemperature & 0x0100)==0x100)
			nTemperature |= 0xFFFFFE00;  // This will set all bits above the 9th bit to ones

		// The 9611 chip provides temperature on a scale that is 2 LSBs for each 0.96 degree.
		// Divide this number in 2 to get a very close approximation (off by ~4%) of the real temp.
		// For higher precision, replace with floating point operations.
		nTemperature = nTemperature / 2;
		*pnSiliconDieTemperature = nTemperature;
	}
	else
		*pnSiliconDieTemperature = 0;

	return(nReturnVal);
}


int max_MAX9611_read_ADC_voltage(u32 unPeripheralAddressI2C, u8 uchBaseRegister, unsigned int *unVoltageADC)
/**
* \brief       Read a 12-bit value from one of the MAX9611's voltage registers.
* \par         Details
*              The votage registers include: Set Voltage, OUT Voltage, Common-mode Voltage and Sense
*              voltage.  The particular register to be read is determined by value on uchBaseRegister parameter.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map
* \param[in]   uchBaseRegister           - register address
* \param[out]  *unVoltageADC             - ADC reading is stored at unVoltageADC
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer=0;
	u8 uchRxBuffer=0;
	int nByteCount=0;
	unsigned int unValue=0;
	int nReturnVal=TRUE;

	uchTxBuffer = uchBaseRegister;  // access the 8 MSBs of OUT voltage register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchTxBuffer, 1, XIIC_STOP);

	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;


	unValue =0;
	unValue = (uchRxBuffer);	// since it gets set here?
	unValue = unValue << 4;		// Move 8 MSBs into bit positions [11:4] of final result


	uchTxBuffer = uchBaseRegister+1;  // increment register address to point to 8 LSBs of OUT voltage register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	unValue = unValue + ((uchRxBuffer & 0x00F0)>> 4); // Combine 8 MSBs and 4 LSBs

	if(nReturnVal==TRUE)
		*unVoltageADC = unValue;
	else
		*unVoltageADC = 0;

	return(nReturnVal);
}


int max_MAX9611_turn_on_2ms_updates(u32 unPeripheralAddressI2C)
/**
* \brief       Set up the MAX9611 for 'fast-read' mode, normal operation (2mS readings).
* \par         Details
*              Sets control register 1 as follows:
* \n           CR1[2:0] = 111 : Read all channels in fast-read mode, sequentially every 2mSec
* \n           CR1[3] = 0 : normal operation (note - "1" = Shutdown mode)
* \n           CR1[4] = 0 : normal operation (note - "1" = Reset if comparator is latched)
* \n           CR1[7:5] = 000 : Normal operation for opamp/comparator
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	int nByteCount=0;
	int nReturnVal=FALSE;

	uchTxBuffer[0] = 0x0A;  // Control register #1
	uchTxBuffer[1] = 0x07;  // Write a (07) into the control register, sets up the onboard ADC to
						// sequentially update all data registers every ~2mS

	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX9611_IIC_ADDRESS,
					(u8 *)&uchTxBuffer, 2, XIIC_STOP);

	if(nByteCount==2)
		nReturnVal=TRUE;

	delay(ABOUT_ONE_SECOND/5);
	return(nReturnVal);
}


int max_MAX5380_change_voltage(u32 unPeripheralAddressI2C,u8 uchRegisterSetting)
/**
* \brief       Set the 8-bit output voltage setting in the MAX5380 DAC (used to set current limit on MAX9611 module)
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchRegisterSetting        - 8-bit voltage output value 
*
* \retval  TRUE if operation succeeded
*/
{
	u8 uchTxBuffer;
	int nReturnVal=TRUE;
	int nByteCount=0;

	uchTxBuffer = uchRegisterSetting;

	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5380_IIC_ADDRESS,
				(u8 *)&uchTxBuffer, 1, XIIC_STOP);

	if(nByteCount!=1)
		nReturnVal=FALSE;

	return(nReturnVal);
}

void max_MAX5487_change_wiper(u32 unPeripheralAddressSPI, u8 uchRegisterSettingA, u8 uchRegisterSettingB, u8 uchChannelA, u8 uchChannelB)
/**
* \brief       Set the wiper position(s) of one or both pots in the MAX5487 dual digital potentiometer.  
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchRegisterSettingA       - 8-bit voltage output value 
* \param[in]   uchRegisterSettingB       - 8-bit voltage output value 
* \param[in]   uchChannelA               - True = update channel A, False = do not update 
* \param[in]   uchChannelB               - True = update channel B, False = do not update 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 outputBuffer[2];
	u8 ReadBuffer[2];

	if(uchChannelA==TRUE)
	{
		outputBuffer[0] = 0x01; // (Write Wiper Register A)
		outputBuffer[1] = uchRegisterSettingA; // Set wiper value
		SpiRW(unPeripheralAddressSPI,0,0,(u8*)&outputBuffer,(u8*)&ReadBuffer,2,0);  // read 2 bytes
	}

	if(uchChannelB==TRUE)
	{
		outputBuffer[0] = 0x02; // (Write Wiper Register A)
		outputBuffer[1] = uchRegisterSettingB; // Set wiper value
		SpiRW(unPeripheralAddressSPI,0,0,(u8*)&outputBuffer,(u8*)&ReadBuffer,2,0);  // read 2 bytes
	}
}

void max_GPIO_bit_bang_SPI_write(XGpio *port, unsigned int unNumberBytes, u8 *pauchBuffer)
/**
* \brief       Performs a SPI write using bit banging.  
* \par         Details
*              This function was written for the MAX5216 DAC but could be used elsewhere as desired.
* \n           The code implements the SPI write with CPOL=0 and CPHA=1
* \n           Polarity = 0 (clock is normally low), Phase = 1 (data is valid & sampled on negative edge)
* \n           XGpio *port is the GPIO port address
*
* \param[in]   *port                 - pointer to the GPIO port instance
* \param[in]   unNumberBytes         - number of bytes to write, typically 1,2 or 3
* \param[in]   *pauchBuffer          - pointer to a buffer containing the data to write
*
* \retval  None
*/
{
	int i=0;
	int j=0;
	u8 uchPortData=0;
	u8 *p;
	u8 uchBuffer;

	XGpio_SetDataDirection(port, 1, 0x00);  // Set all the GPIO to outputs


	// First, set the clock and data lines low, and the SS# line high
		// GPIO[3:0] = {SCK, CLR#, MOSI, SS#} // default 'off' is 4'b0101 = 0x05
	uchPortData = 0x05;
	XGpio_DiscreteWrite(port, 1, uchPortData);  // The (1) means write to the only port (#1) on the uBlaze GPIO port

	// Next, assert SS#
	uchPortData=0x04;
	XGpio_DiscreteWrite(port, 1, uchPortData);

	p = pauchBuffer;	// setup a local pointer so as to preserve the original pauchBuffer address
	p--; 				// pre-decrement for the pre-increment in the loop

	for(i=0;i<unNumberBytes;i++)
	{
		p++;
		uchBuffer = *p;		// get a local copy of the data in the buffer;
		for(j=7;j>=0;j--)
		{
			// Send clock high
			uchPortData |= BIT_BANG_SPI_CLK;
			// Set data bit by shifting data, masking and seeing if value is equal to 1'b1
			if(((uchBuffer >> j) & 0x01)==0x01)
				uchPortData |= BIT_BANG_SPI_MOSI;
			else
				uchPortData &= ~BIT_BANG_SPI_MOSI;
			XGpio_DiscreteWrite(port, 1, uchPortData);
		//	delay(1);  // small delay

			uchPortData &= ~BIT_BANG_SPI_CLK;// Clock in data via negative edge
			XGpio_DiscreteWrite(port, 1, uchPortData);
			//delay(1);  // small delay
		}
	}

	// Cleanup the port
	// Set the clock and data lines low, and the SS# line high
	// GPIO[3:0] = {SCK, CLR#, MOSI, SS#} // default 'off' is 4'b0101 = 0x05
	uchPortData = 0x05;
	XGpio_DiscreteWrite(port, 1, uchPortData);  // The (1) means write to the only port (#1) on the uBlaze GPIO port
}

int max_MAX11205_GPIO_SPI_read(XGpio *port, u8 uchUseCalibrationMode)
/**
* \brief       Performs a SPI read using bit banging. Written specifically for the MAX11205 ADC.
*
* \par         Description
*              The MAX11205 sets the RDY# line high during sampling, then low when a sample is available.
*              The function waits for that transition, then provides the appropriate number of clocks
*              to the MAX11205 to complete the read.  Note that the 11205 is a 16 bit device, but requires
*              9 additional clocks (25 in total) to complete a read cycle
*              An optional self-calibration mode can be selected by setting uchUseCalibrationMode==TRUE
*              which will send a 26th clock to the MAX11205.
*
* \param[in]   *port                       - pointer to the GPIO port instance
* \param[in]   uchUseCalibrationMode       - TRUE = perform function in self-calibration mode. 
*
* \retval      adcValue                    - 16-bit value from ADC conversion
*/
{
	int i=0;
	u8 uchPortWriteData=0;
	u8 uchPortReadData=0;
	int adcValue=0;
	int nClockCount=8;

	XGpio_SetDataDirection(port, 1, 0x04);  // Set MISO to be an input.  All the rest (including SCLK) are outputs

	delay(10);
	// First, set the clock and data lines low, and the SS# line high
		// GPIO[3:0] = {SCK, CLR#, MOSI, SS#} // default 'off' is 4'b0101 = 0x05
	delay(10);
	uchPortWriteData = 0;
	XGpio_DiscreteWrite(port, 1, uchPortWriteData);  // The (1) means write to the only port (#1) on the uBlaze GPIO port
	delay(10);

	// Wait until RDY# goes high
	uchPortReadData = XGpio_DiscreteRead(port,1);
	while((uchPortReadData & BIT_BANG_SPI_MISO)==0x00)  // keep looping while low
	{
		uchPortReadData = XGpio_DiscreteRead(port,1);
	}

	// Now, wait until RDY# goes low
	uchPortReadData = XGpio_DiscreteRead(port,1);
	while((uchPortReadData & BIT_BANG_SPI_MISO)==BIT_BANG_SPI_MISO)
	{
		uchPortReadData = XGpio_DiscreteRead(port,1);
	}

	// Now, send in clocks.
	for(i=15;i>=0;i--)
	{
		// Send clock high
		uchPortWriteData |= BIT_BANG_SPI_CLK;
		XGpio_DiscreteWrite(port, 1, uchPortWriteData);

		delay(10);  // small delay, then read
		uchPortReadData = XGpio_DiscreteRead(port,1);

		// Shift the GPIO read data to the lowest bit, mask off all the other bits, then shift i (15:0)
		// number of bits to set the appropriate bit in the
		if(((uchPortReadData >> 2) & 0x01)==0x01) // the serial bit is a one, set the bit
		{
			if(i==15)
				adcValue = 0xFFFF8000;  // extend the sign of the 2s complement number to bits 31..16, and set bit 15 = 1
			else
			    adcValue |= (int)(1 << i);  // bit shift
		}
		uchPortWriteData &= ~BIT_BANG_SPI_CLK;// Clock in data via negative edge
		XGpio_DiscreteWrite(port, 1, uchPortWriteData);
		delay(10);
	}

	// The 11205 device requires 25 clocks total to complete a read.
	// We have already sent (16) clocks, so send 9 more clocks.
	// (If self-calibration mode set then we send 10 more clocks.)
	if(uchUseCalibrationMode==TRUE)
		nClockCount = 9;
	else
		nClockCount=8;
	for(i=nClockCount;i>=0;i--)
	{
		// Send clock high
		uchPortWriteData |= BIT_BANG_SPI_CLK;
		XGpio_DiscreteWrite(port, 1, uchPortWriteData);

		delay(10);  // small delay, then read
		uchPortWriteData &= ~BIT_BANG_SPI_CLK;// Clock in data via negative edge
		XGpio_DiscreteWrite(port, 1, uchPortWriteData);
		delay(10);
	}
	return(adcValue);
}

void max_MAX5216_set_output_voltage(XGpio *pmodPort, u16 value)
/**
* \brief       Set the 16-bit output voltage on the MAX5216 DAC.
* \par         Details
*              This function uses the max_GPIO_bit_bang_SPI_write function to bit-bang the value into
*              the MAX5216.
*
* \param[in]   *pmodPort             - pointer to the pmodPort port instance
* \param[in]   value                 - 16-bit value to write to the MAX5216 output register
*
* \retval      None
*/
{
	u8 auchOutputBuffer[3];

	auchOutputBuffer[0] = 0x40; // (Bits 23, 22 set to 2'b01 to write through to DAC
	auchOutputBuffer[0] += (u8)(((value & 0xff00) >> 10) & 0xFF);  // Bit 15 ..B10
	auchOutputBuffer[1] = (u8)(((value & 0x03FC) >> 2) & 0xFF); // Bit 9..bit2
	auchOutputBuffer[2] = (u8)(((value & 0x0003) << 6) & 0xFF); // Bits 2..0

	// This function uses Polarity = 0 (clock is normally low), Phase = 1 (data is valid & sampled on negative edge)
	max_GPIO_bit_bang_SPI_write(pmodPort, 3, auchOutputBuffer);
}


int max_MAX5825_set_internal_reference(u32 unPeripheralAddressI2C, u8 uchReferenceCommand)
/**
* \brief       Initialize the internal voltage reference in the MAX5825
* \par         Details
*              Only the 3 LSBs of the uchReference command are used (sent in bit[18:16] of command)
* \n           uchReferenceCommand[2] = 0: Ref power off, 1: Ref power on (normal operating mode)
* \n           uchReferenceCommand[1:0] = 00: ext ref, 01: internal 2.500V, 10: internal 2.048V, 11: internal 4.096V
* \n           Note that int ref of 4.096V requires VCC = 4.5V min
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchReferenceCommand       - specifies voltage reference setting desired 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[3];
	int nByteCount=0;
	int nReturnVal;
	u8 uchCommandToSend=0;

	uchCommandToSend = uchReferenceCommand;
	uchCommandToSend &= 0x07;  // mask off all but 3 LSBs
	uchCommandToSend |= 0x20;  // set the REF command bit

	uchTxBuffer[0] = uchCommandToSend;  // REF command is 0x2-  where bit[3]=0 and bit[2:0] is 0-7
	uchTxBuffer[1] = 0x00;
	uchTxBuffer[2] = 0x00;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5825_IIC_ADDRESS,
								(u8 *)&uchTxBuffer, 3, XIIC_STOP);

	if(nByteCount!=3)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX5825_send_sw_clear(u32 unPeripheralAddressI2C)
/**
* \brief       Send a software clear (SW_CLEAR) command to the MAX5825.
* \par         Details
*              All CODE and DAC registers are cleared to their default values.
* \n           Sends the three byte command = 0x349630
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[3];
	int nByteCount=0;
	int nReturnVal;

	// Send SW CLEAR (0x34 0x96 0x30)
	uchTxBuffer[0] = 0x34;
	uchTxBuffer[1] = 0x96;
	uchTxBuffer[2] = 0x30;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5825_IIC_ADDRESS,
			(u8 *)&uchTxBuffer, 3, XIIC_STOP);

	if(nByteCount!=3)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX5825_send_sw_reset(u32 unPeripheralAddressI2C)
/**
* \brief       Send a software reset (SW_RESET) command to the MAX5825.
* \par         Details
*              All CODE, DAC and control registers are cleared to their power-on values.
*              Sends the three byte command = 0x359630
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[3];
	int nByteCount=0;
	int nReturnVal;

	// Send SW RESET (0x35 96 30)
	uchTxBuffer[0] = 0x35;  // CODE command + All 8 DACs address
	uchTxBuffer[1] = 0x96;
	uchTxBuffer[2] = 0x30;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5825_IIC_ADDRESS,
			(u8 *)&uchTxBuffer, 3, XIIC_STOP);

	if(nByteCount!=3)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX5825_set_default_DAC_settings(u32 unPeripheralAddressI2C,u8 uchMask, u8 uchDefaultCodeSetting)
/**
* \brief       Send a software clear (SW_CLEAR) command to the MAX5825.  
* \par         Details
*              All CODE registers are set to 1 of 5 possible default states, based on uchDefaultCodeSetting:
* \n           MAX5825_DEFAULT_M_ZNOT: set dac's CODE register based on M/Z pin setting
* \n           MAX5825_DEFAULT_ZERO: set dac's CODE register to zero
* \n           MAX5825_DEFAULT_MID: set dac's CODE register to mid scale
* \n           MAX5825_DEFAULT_FULL: set dac's CODE register to full scale
* \n           MAX5825_DEFAULT_RETURN: set dac's code register = RETURN register
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchMask                   - selects which DACs to affect, 1 bit per DAC 
* \param[in]   uchDefaultCodeSetting     - determines which default state to set dac's CODE register 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[3];
	int nByteCount=0;
	int nReturnVal;

	// Setup and send the DEFAULT command
	uchTxBuffer[0] = 0x60;  	//The base DEFAULT command is 0x60
	uchTxBuffer[1] = uchMask;	// bits[7:0] of uchMask represent {DAC7, DAC6, ... , DAC1, DAC0}

	// Send one of the (5) possible default DAC settings
	switch(uchDefaultCodeSetting)
	{
		case MAX5825_DEFAULT_M_ZNOT:
			uchTxBuffer[2] = MAX5825_DEFAULT_M_ZNOT;
			break;
		case MAX5825_DEFAULT_ZERO:
			uchTxBuffer[2] = MAX5825_DEFAULT_ZERO;
			break;
		case MAX5825_DEFAULT_MID:
			uchTxBuffer[2] = MAX5825_DEFAULT_MID;
			break;
		case MAX5825_DEFAULT_FULL:
			uchTxBuffer[2] = MAX5825_DEFAULT_FULL;
			break;
		case MAX5825_DEFAULT_RETURN:
			uchTxBuffer[2] = MAX5825_DEFAULT_RETURN;
			break;
		default:
			uchTxBuffer[2] = MAX5825_DEFAULT_ZERO;
			break;
	}

	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5825_IIC_ADDRESS,
			(u8 *)&uchTxBuffer, 3, XIIC_STOP);

	if(nByteCount!=3)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX5825_initialize(u32 unPeripheralAddressI2C)
/**
* \brief       Set the MAX5825 dac CODE registers equal to their respective RETURN registers and 
*              set the voltage reference = internal 2.500V
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal=TRUE;
	if(max_MAX5825_send_sw_reset(unPeripheralAddressI2C)!=TRUE)
		nReturnVal=FALSE;

	delay(100);

	// Set all dac CODE registers = respective RETURN register settings
	if(max_MAX5825_set_default_DAC_settings(unPeripheralAddressI2C,0xFF, MAX5825_DEFAULT_RETURN)!=TRUE)
			nReturnVal=FALSE;

		delay(100);

	// Set internal reference to 2.500V
	if(max_MAX5825_set_internal_reference(unPeripheralAddressI2C, MAX5825_2500_MV_REFERENCE)!=TRUE)
		nReturnVal=FALSE;

	delay(100);

	return(nReturnVal);
}

int max_MAX5825_set_CODE_register(u32 unPeripheralAddressI2C,u8 uchChannelToSet, unsigned int unRegisterSetting)
/**
* \brief       Set the CODE register for selected dac channel(s) within the MAX5825.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchChannelToSet           - dac channel# (0-7) to load (8=all) 
* \param[in]   unRegisterSetting         - value to load into the CODE register 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[3];
	int nByteCount=0;
	unsigned int unDACValue=0;
	u8 uchChannel=0x00;
	int nReturnVal=TRUE;

	if(unRegisterSetting >= 4096)
		unDACValue = 4095;
	else
		unDACValue = unRegisterSetting;

	if(uchChannelToSet > 0x08)	
		uchChannel = 0x08;
	else
		uchChannel = uchChannelToSet;

	// Send the CODE command
	uchTxBuffer[0] = 0x80;  // The base CODEn command is 0x80
	uchTxBuffer[0] |= uchChannel;  // Set channel #0-7, or 8=all
	uchTxBuffer[1] = ((unDACValue & 0x0FF0) >> 4);  // MSB data
	uchTxBuffer[2] = ((unDACValue & 0x000F) << 4);	// LSB DATA (aligned to upper 4 bits)
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5825_IIC_ADDRESS,
						(u8 *)&uchTxBuffer, 3, XIIC_STOP);
	if(nByteCount!=3)
				nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX5825_set_RETURN_register(u32 unPeripheralAddressI2C,u8 uchChannelToSet, unsigned int unRegisterSetting)
/**
* \brief       Set the RETURN register for selected dac channel(s) within the MAX5825.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchChannelToSet           - dac channel# (0-7) to load (8=all) 
* \param[in]   unRegisterSetting         - value to load into the RETURN register 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[3];
	int nByteCount=0;
	unsigned int unDACValue=0;
	u8 uchChannel=0x00;
	int nReturnVal=TRUE;

	if(unRegisterSetting >= 4096)
		unDACValue = 4095;
	else
		unDACValue = unRegisterSetting;

	if(uchChannelToSet > 0x08)
		uchChannel = 0x08;
	else
		uchChannel = uchChannelToSet;

	// Send the RETURN command
	uchTxBuffer[0] = 0x70;  // The base RETURNn command is 0x70
	uchTxBuffer[0] |= uchChannel;  // Set channel #0-7, or 8=all
	uchTxBuffer[1] = ((unDACValue & 0x0FF0) >> 4);  // MSB data
	uchTxBuffer[2] = ((unDACValue & 0x000F) << 4);	// LSB DATA (aligned to upper 4 bits)
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5825_IIC_ADDRESS,
						(u8 *)&uchTxBuffer, 3, XIIC_STOP);
	if(nByteCount!=3)
				nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX5825_LOAD_DAC_from_CODE_register(u32 unPeripheralAddressI2C,u8 uchChannelToSet)
/**
* \brief       Set the DAC register from the CODE register for selected dac channel(s) within the MAX5825.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchChannelToSet           - dac channel# (0-7) to load (8=all) 
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[3];
	int nByteCount=0;
	u8 uchChannel=0x00;
	int nReturnVal=TRUE;

	if(uchChannelToSet > 0x08)
		uchChannel = 0x08;
	else
		uchChannel = uchChannelToSet;

	// Send the LOAD command
	uchTxBuffer[0] = 0x90;  // The base LODEn command is 0x90
	uchTxBuffer[0] |= uchChannel; 	// Set channel #0-7, or 8=all
	uchTxBuffer[1] = 0x00;  // NULL - doesn't matter, data is ignored.
	uchTxBuffer[2] = 0x00;  // NULL - doesn't matter, data is ignored.
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX5825_IIC_ADDRESS,
						(u8 *)&uchTxBuffer, 3, XIIC_STOP);

	if(nByteCount!=3)
			nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX5825_set_DAC_12_bit_value(u32 unPeripheralAddressI2C,u8 uchChannelToSet, unsigned int unRegisterSetting)
/**
* \brief       Set the CODE register for the specified channel within the MAX5825 to the value unRegisterSetting,
*              then move that value into the DAC register.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchChannelToSet           - dac channel# (0-7) to load (8=all) 
* \param[in]   unRegisterSetting         - value to load into the CODE register 
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal=TRUE;
	if(max_MAX5825_set_CODE_register(unPeripheralAddressI2C,uchChannelToSet, unRegisterSetting)!=TRUE)
		nReturnVal=FALSE;

	if(max_MAX5825_LOAD_DAC_from_CODE_register(unPeripheralAddressI2C,uchChannelToSet)!=TRUE)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX7304_get_direction(u32 unPeripheralAddressI2C, u8 *puchHighByte, u8 *puchLowByte)
/**
* \brief       Retrieve the data direction bits (0=in or 1=out) for all 16 ports in the MAX7304.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *puchHighByte             - data direction bits for Port[7:0] are stored at puchHighByte
* \param[out]  *puchLowByte              - data direction bits for Port[15:8] are stored at puchLowByte
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	u8 uchRxBuffer=0;
	int nReturnVal=TRUE;
	int nByteCount=0;

	// Get the GPIO direction mode register (0x34).  0 is input.  1 is output,
	uchTxBuffer[0] = 0x34;  // MAX7304 GPIO Direction Register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	else
		*puchLowByte = uchRxBuffer;

	// Get the GPIO direction mode register (0x35).  0 is input.  1 is output,
	uchTxBuffer[0] = 0x35;  // MAX7304 GPIO Direction Register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	else
		*puchHighByte = uchRxBuffer;

	return(nReturnVal);
}

int max_MAX7304_set_direction(u32 unPeripheralAddressI2C, u8 uchHighByte, u8 uchLowByte)
/**
* \brief       Set the data direction bits (input or output) for the MAX7304 GPIO.  
* \par         Details
*              Generally, if only a single (or few bits) need to be changed, then the function
*              max_MAX7304_get_direction should be called to retrieve the present MSB and LSB byte
*              direction settings.  Those settings should be modified and then this function used to set
*              the new values.
* \n           WARNING: On the MAX7304 peripheral module the upper 4 ports [15:12] are connected to
*              LEDs which do not have current limiting resistors.  If these ports are set to standard
*              logic outputs with outputs low, excessively high currents could result, possibly
*              destroying the LEDs.  It is recommended to keep those 4 ports set as inputs.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *puchHighByte             - values for Port[7:0] data direction bits are stored at puchHighByte
* \param[out]  *puchLowByte              - values for Port[15:8] data direction bits are stored at puchLowByte
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	int nReturnVal=TRUE;
	int nByteCount=0;

	uchTxBuffer[0] = 0x34;  // MAX7304 GPIO Direction Register for Ports[7:0]
	uchTxBuffer[1] = uchLowByte;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	uchTxBuffer[0] = 0x35;  // MAX7304 GPIO Direction Register for Ports[15:8]
	uchTxBuffer[1] = uchHighByte;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;
	
	return(nReturnVal);
}

int max_MAX7304_enable(u32 unPeripheralAddressI2C, u8 uchValue)
/**
* \brief       Enable the GPIO operation for the MAX7304.  
* \par         Details
*              This function sets or clears bit4 in the GPIO Global Configuration Register in the
*              MAX7304.  Setting to "1" enables normal GPIO operation.  PWM, constant current circuits
*              and GPIOs are enabled.  Cleared to "0", PWM, constant current circuits and GPIs are shut
*              down.  GPO values depend on their settings.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchValue                  - "1" to set bit4, "0" to clear bit4.
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	int nReturnVal=TRUE;
	int nByteCount=0;

	// Set bit 4 in the GPIO Global Configuration Register 1=enabled, 0 = disabled (limited functionality)
	uchTxBuffer[0] = 0x40;
	if(uchValue==TRUE)
		uchTxBuffer[1] = 0x10;
	else
		uchTxBuffer[1] = 0x00;

	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX7304_get_debounce_value(u32 unPeripheralAddressI2C, u8 *uchValue)
/**
* \brief       Retrieve the debounce time for the MAX7304 inputs
* \par         Details
*              Debounce time = (retrieved value + 9) mSec.  Max value is 40mSec.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *uchValue                 - value for debounce time is stored at uchValue
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	u8 uchRxBuffer[2];
	int nReturnVal=TRUE;
	int nByteCount=0;

	// I2C reg addr for debounce time is 0x42
	uchTxBuffer[0] = 0x42;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	uchRxBuffer[0] = 0x00;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
			nReturnVal=FALSE;

	if(nReturnVal==TRUE)
		*uchValue = (uchRxBuffer[0] & 0x1F);		// only the 5 LSBs are used to set debounce value

	return(nReturnVal);
}

int max_MAX7304_set_debounce_value(u32 unPeripheralAddressI2C, u8 uchValue)
/**
* \brief       Set the debounce time for the MAX7304 inputs
* \par         Details
*              Debounce time = (retrieved value + 9) mSec.  Only the 5 LSBs are recognized; max value
*              for debounce time is 40mSec.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchValue                  - value for debounce time
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	int nReturnVal=TRUE;
	int nByteCount=0;

	// I2C reg addr for debounce time is 0x42
	uchTxBuffer[0] = 0x42;
	if(uchValue > 0x1F)		// Cap the value at 5 LSBs
		uchTxBuffer[1]=0x1F;
	else
		uchTxBuffer[1]=uchValue;

	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_MAX7304_get_input(u32 unPeripheralAddressI2C, u8 *puchHighByte, u8 *puchLowByte)
/**
* \brief       Retrieve the GPI values for all ports (MSB and LSB).
* \par         Details
*              Per the MAX7304 datasheet, GPIO configured as outputs will not necessarily
*              return the correct values.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *puchHighByte             - value for Port[7:0] inputs is stored at puchHighByte
* \param[out]  *puchLowByte              - value for Port[15:8] inputs is stored at puchLowByte
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	u8 uchRxBuffer;
	int nReturnVal=TRUE;
	int nByteCount=0;

	// Get the value of the low byte (7..0)
	uchTxBuffer[0] = 0x3A;  // MAX7304 GPIO Value Register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	uchRxBuffer = 0x00;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	else
		*puchLowByte = uchRxBuffer;


	// Get the value of the high byte (15..8)
	uchTxBuffer[0] = 0x3B;  // MAX7304 GPIO Value Register
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	uchRxBuffer = 0x00;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	else
		*puchHighByte = uchRxBuffer;

	return(nReturnVal);
}

int max_MAX7304_set_output(u32 unPeripheralAddressI2C, u8 uchHighByte, u8 uchLowByte)
/**
* \brief       Set the MSB and LSB bytes of the MAX7304 GPO.  
* \par         Details
*              GPIO set to input will not be affected by this function call.
* \n           WARNING: On the MAX7304 peripheral module the upper 4 ports [15:12] are connected to
*              LEDs which do not have current limiting resistors.  If these ports are set to standard
*              logic outputs with outputs low, excessively high currents could result, possibly
*              destroying the LEDs.  It is recommended to keep those 4 ports set as inputs.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchHighByte               - value to write to Port[15:8] outputs
* \param[in]   uchLowByte                - value to write to Port[7:0] outputs
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	int nReturnVal=TRUE;
	int nByteCount=0;

	uchTxBuffer[0] = 0x3A;  		// MAX7304 GPIO bits [7:0] Value Register
	uchTxBuffer[1] = uchLowByte;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	uchTxBuffer[0] = 0x3B;  		// MAX7304 GPIO bits [15:8] Value Register
	uchTxBuffer[1] = uchHighByte;
	nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer[0], 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	delay(50000);
	return(nReturnVal);
}

int max_MAX7304_configure_PWM(u32 unPeripheralAddressI2C,u8 uchPwm1, u8 uchPwm2, u8 uchPwm3, u8 uchPwm4)
/**
* \brief       Set the PWM register value for the LED driver PWMs in the MAX7304.  
* \par         Details
*              uchPwm values can range from 0..255 and represent 0 to 255/256ths of a fully active channel.  This
*              function also sets up the LED drive enable register (0x31) to enable all 4 LED ports (Port12-Port15)
*              for LED output and sets the Global Configuration Register (0x40) to enable PWM control and constant 
*              current drivers.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchPwm1                   - value to write to PWM register for port 12
* \param[in]   uchPwm2                   - value to write to PWM register for port 13
* \param[in]   uchPwm3                   - value to write to PWM register for port 14
* \param[in]   uchPwm4                   - value to write to PWM register for port 15
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[2];
	int nReturnVal=TRUE;
	int nByteCount=0;

	// Configure MAX7304 board
		uchTxBuffer[0] = 0x50;  // LED Port12 PWM ratio
		uchTxBuffer[1] = uchPwm1;    // Decimal value is ratio of x / 256ths brightness
		nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
				(u8 *)&uchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
		delay(50000);

		uchTxBuffer[0] = 0x51;  // LED Port13 PWM ratio
		uchTxBuffer[1] = uchPwm2;    // Decimal value is ratio of x / 256ths brightness
		nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
				(u8 *)&uchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
		delay(50000);

		uchTxBuffer[0] = 0x52;  // LED Port14 PWM ratio
		uchTxBuffer[1] = uchPwm3;    // Decimal value is ratio of x / 256ths brightness
		nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
				(u8 *)&uchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
		delay(50000);

		uchTxBuffer[0] = 0x53;  // LED Port15 PWM ratio
		uchTxBuffer[1] = uchPwm4;    // Decimal value is ratio of x / 256ths brightness
		nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
				(u8 *)&uchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
		delay(50000);

		uchTxBuffer[0] = 0x31;  // LED Driver enable register
		uchTxBuffer[1] = 0x0F;  // Bits [3:0] turn on the LED functionality (disable GPIO functionality)
		nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
		delay(50000);

		uchTxBuffer[0] = 0x40;  // GPIO Global config register
		uchTxBuffer[1] = 0x10;  // Turn on the PWM circuits, constant current circuits, etc
		nByteCount = XIic_Send(unPeripheralAddressI2C, MAX7304_IIC_ADDRESS,
			(u8 *)&uchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
		delay(50000);
		return(nReturnVal);
}

// This function retrieves the floating point (degrees C) temperature from the 3231M
int max_DS3231M_get_temp(u32 unPeripheralAddressI2C, float *fTemp)
/**
* \brief       Retrieve the floating point (degrees C) temperature from the 3231M.  
* \par         Details
*              uchPwm values can range from 0..255 and represent 0 to 255/256ths of a fully active channel.  This
*              function also sets up the LED drive enable register (0x31) to enable all 4 LED ports (Port12-Port15)
*              for LED output and sets the Global Configuration Register (0x40) to enable PWM control and constant 
*              current drivers.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *fTemp		             - value for temperature is stored at fTemp
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer;
	u8 auchRxBuffer[8];
	int nReturnVal = TRUE;
	int nByteCount=0;
	int nTemporaryTemperature=0;
	float fTemporaryTemperature=0.0f;

	// DS3231M Temperature register(s) begin at 0x11
	uchTxBuffer = 0x11;

	//Send the register address.  This performs an I2C [START/CHIP_ADDRESS[W]], writes the register address, then sends a [STOP].
	//Find these calls in xiic_l.c.
	nByteCount = XIic_Send( unPeripheralAddressI2C, DS3231M_IIC_ADDRESS, &uchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal = FALSE;
	else
	{
		//We set the DS3231M register pointer in the previous call.  Now read from it.
		//This call sends a [START/CHIP_ADDRESS[R]], reads out 7 bytes, and issues a [STOP].
		nByteCount = XIic_Recv( unPeripheralAddressI2C, DS3231M_IIC_ADDRESS, (u8*)&auchRxBuffer[0], 0x02, XIIC_STOP );
		if(nByteCount!=2)
			nReturnVal = FALSE;
		else
		{
			if((auchRxBuffer[0] & 0x80)==0x00)  // positive number
			{
				nTemporaryTemperature = ((int)(auchRxBuffer[0] & 0x7F)) << 2;
				nTemporaryTemperature += ((int)(auchRxBuffer[1] & 0xC0)) >> 6;
				fTemporaryTemperature = (float)nTemporaryTemperature;
				fTemporaryTemperature /= 4.0f;
			}
			else
			{
				nTemporaryTemperature = ((int)(~auchRxBuffer[0] & 0x7F)) << 2;
				nTemporaryTemperature += ((int)(~auchRxBuffer[1] & 0xC0)) >> 6;
				nTemporaryTemperature += 1;
				fTemporaryTemperature = (float)nTemporaryTemperature;
				fTemporaryTemperature *= -1.0f;
				fTemporaryTemperature /= 4.0f;
			}
			*fTemp = fTemporaryTemperature;
		}
	}
	return(nReturnVal);
}

int max_DS3231M_get_time(u32 unPeripheralAddressI2C, struct maximDateTime *t)
/**
* \brief       Retrieves all major date/time values from the DS3231M.  
* \par         Details
*              This function retrieves all (7) major date/time registers inside the DS3231M 
*              and populates a date/time structure with.  The structure includes the following:
* \n           u8 uchHour;
* \n           u8 uchMinute;
* \n           u8 uchSecond;
* \n           u8 uchMonth;
* \n           u8 uchDate;
* \n           u8 uchYear;
* \n           u8 uchDow;
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *t                        - value for temperature is stored at t (structure)
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer;
	u8 auchRxBuffer[8];
	int nReturnVal = TRUE;
	int nByteCount=0;


	//TempPtr[0] will be our register address.
	uchTxBuffer = 0;

	//Send the register address.  This performs an I2C [START/CHIP_ADDRESS[W]], writes the register address, then sends a [STOP].
	//Find these calls in xiic_l.c.
	nByteCount = XIic_Send( unPeripheralAddressI2C, DS3231M_IIC_ADDRESS, &uchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal = FALSE;
	else
	{
		//We set the DS3231M register pointer in the previous call.  Now read from it.
		//This call sends a [START/CHIP_ADDRESS[R]], reads out 7 bytes, and issues a [STOP].
		nByteCount = XIic_Recv( unPeripheralAddressI2C, DS3231M_IIC_ADDRESS, (u8*)&auchRxBuffer[0], 0x07, XIIC_STOP );
		if(nByteCount!=7)
			nReturnVal = FALSE;
		else
		{
			//See DS3231M datasheet to see where these assignments came from.
			// many of the values are provided in Binary Coded Decimal
			t->uchSecond 	= (((auchRxBuffer[0] & 0x70) >> 4)*10) + (auchRxBuffer[0] & 0x0F);
			t->uchMinute 	= (((auchRxBuffer[1] & 0x70) >> 4)*10) + (auchRxBuffer[1] & 0x0F);
			if((auchRxBuffer[2] & 0x40)==0x40) // 12 hr mode
				t->uchHour		= (((auchRxBuffer[2] & 0x20) >> 5)*12) + (((auchRxBuffer[2] & 0x10) >> 4)*10) + (auchRxBuffer[2] & 0x0F);
			else  // 24 hrs mode
				t->uchHour		= (((auchRxBuffer[2] & 0x20) >> 5)*20) + (((auchRxBuffer[2] & 0x10) >> 4)*10) + (auchRxBuffer[2] & 0x0F);
			t->uchDow 		= auchRxBuffer[3] & 0x07;
			t->uchDate 		= (((auchRxBuffer[4] & 0x30) >> 4)*10) + (auchRxBuffer[4] & 0x0F);
			t->uchMonth 	= (((auchRxBuffer[5] & 0x10) >> 4)*10) + (auchRxBuffer[5] & 0x0F);
			t->uchYear		= (((auchRxBuffer[6] & 0xF0) >> 4)*10) + (auchRxBuffer[6] & 0x0F);
		}
	}
	return(nReturnVal);
}

int max_DS3231M_set_time(u32 unPeripheralAddressI2C, struct maximDateTime *t)
/**
* \brief       Store values from date/time structure into DS3231M registers.
* \par         Details
*              This function loads values for the major date/time variables into the DS3231M 
*              registers.  The input structure includes the following:
* \n           u8 uchHour;
* \n           u8 uchMinute;
* \n           u8 uchSecond;
* \n           u8 uchMonth;
* \n           u8 uchDate;
* \n           u8 uchYear;
* \n           u8 uchDow;
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   *t                        - points to structure containing values to load into DS3231M
*
* \retval      TRUE if operation succeeded
*/
{
	u8 uchTxBuffer[10];
	int nReturnVal = TRUE;
	int nByteCount=0;

	// Set the base register to be 0x00 (the seconds register)
	// all other addresses subsequent to this will be automatically
	// incremented when writing
	uchTxBuffer[0] = 0x00;

	// Seconds (Register 0x00)
	uchTxBuffer[1] = ((t->uchSecond / 10) & 0x07) << 4;
	uchTxBuffer[1] += t->uchSecond - ((t->uchSecond / 10) * 10);

	// Minutes (Register 0x01)
	uchTxBuffer[2] = ((t->uchMinute / 10) & 0x07) << 4;
	uchTxBuffer[2] += t->uchMinute - ((t->uchMinute / 10) * 10);

	// Hours (Register 0x02)
	if(t->uchHour >=20)
	{
		uchTxBuffer[3]= 0x20;  // set 24 hrs mode + 20 hrs bit
		uchTxBuffer[3] += (t->uchHour - 20);
	}
	else if(t->uchHour>=10 && t->uchHour<20)
	{
		uchTxBuffer[3]= 0x10;  // set 24 hrs mode + 10 hrs bit
		uchTxBuffer[3] += (t->uchHour - 10);
	}
	else
	{
		uchTxBuffer[3] = 0x00;
		uchTxBuffer[3] += (t->uchHour & 0x0F);
	}

	// Day of Week (Register 0x03)
	uchTxBuffer[4] = t->uchDow;

	// Date (Day of month) (Register 0x04)
	uchTxBuffer[5] = ((t->uchDate / 10) & 0x03) << 4;
	uchTxBuffer[5] += t->uchDate - ((t->uchDate / 10) * 10);

	// Month (Register 0x05)
	uchTxBuffer[6] = ((t->uchMonth / 10) & 0x01) << 4;
	uchTxBuffer[6] += t->uchMonth - ((t->uchMonth / 10) * 10);

	// Year (Register 0x06)
	uchTxBuffer[7] = ((t->uchYear / 10) & 0x0F) << 4;
	uchTxBuffer[7] += t->uchYear - ((t->uchYear / 10) * 10);

	//Send the register address.  This performs an I2C [START/CHIP_ADDRESS[W]], writes the register address, then sends a [STOP].
	//Find these calls in xiic_l.c.
	nByteCount = XIic_Send( unPeripheralAddressI2C, DS3231M_IIC_ADDRESS, &uchTxBuffer[0], 8, XIIC_STOP);
	if(nByteCount!=8)
		nReturnVal = FALSE;

	return(nReturnVal);
}

int max_DS3231M_32kHz(u32 unPeripheralAddressI2C, u8 uchEnable)
/**
* \brief       Enable / disable the 32kHz signal via the DS3231M_EN32KHZ bit.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   uchEnable                 - True = enable 32kHz output, False = disable 32 kHz output
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal = TRUE;
	u8 auchTxBuffer[2];
	u8 auchRxBuffer[2];
	u8 uchStatusRegister=0;
	int nByteCount=0;

	// Read the existing status register value
	auchTxBuffer[0] = DS3231M_STATUS_REG;  // Main configuration register
	auchTxBuffer[1] = 0x00;  // Ambient light sensor mode with (default) factory trimmed gains
	nByteCount = XIic_Send(unPeripheralAddressI2C, DS3231M_IIC_ADDRESS,
									(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS3231M_IIC_ADDRESS,
										(u8 *)&auchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
			nReturnVal=FALSE;

	if(nReturnVal==TRUE)
	{
		uchStatusRegister = auchRxBuffer[0];
		if(uchEnable==TRUE)
			uchStatusRegister |= DS3231M_EN32KHZ;
		else
			uchStatusRegister &= ~DS3231M_EN32KHZ;

		auchTxBuffer[0] = DS3231M_STATUS_REG;  // DS3231M Status Register
		auchTxBuffer[1] = uchStatusRegister;   // Value to write to register (32kHz enabled or disabled)
			nByteCount = XIic_Send(unPeripheralAddressI2C, DS3231M_IIC_ADDRESS,
											(u8 *)&auchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
	}

	return(nReturnVal);
}

int max_DS3231M_get_dow(u32 unPeripheralAddressI2C, u8 *uchDayOfWeek)
/**
* \brief       Retrieve the Day of the Week from DS3231M.  
* \par         Details
*              Day of the Week is returned as an integer value: Sunday=0, Monday=1, ... , Sat=6
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *uchDayOfWeek		     - value for Day of the Week is stored at uchDayOfWeek
*
* \retval      TRUE if operation succeeded
*/
{
	int nReturnVal = TRUE;
	u8 auchTxBuffer[2];
	u8 auchRxBuffer[2];
	int nByteCount=0;


	// Read the existing status register value
	auchTxBuffer[0] = DS3231M_DOW_REGISTER;  // Main configuration register
	auchTxBuffer[1] = 0x00;
	nByteCount = XIic_Send(unPeripheralAddressI2C, DS3231M_IIC_ADDRESS,
									(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS3231M_IIC_ADDRESS,
										(u8 *)&auchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
			nReturnVal=FALSE;

	if(nReturnVal==TRUE)
	{
		auchRxBuffer[0] = auchRxBuffer[0] & 0x03;
		*uchDayOfWeek = auchRxBuffer[0];
	}

	return(nReturnVal);
}

void max_MAX4824_set_all_active(XGpio *port, int nPsaveActive)
/**
* \brief       Set all MAX4824 ports active (on).  
* \par         Details
*              All MAX4824 functions are controlled by an 8-bit parallel GPIO port driving its control
*              pins.  This function sets all outputs active by driving the SETbar pin low.  It also
*              allows for PSAVE mode to be enabled/disabled.
*
* \param[in]   *port                 - pointer to the GPIO port instance
* \param[in]   nPsaveActive		     - True = PSAVE mode on, False = PSAVE mode off
*
* \retval      None
*/
{
		u8 uchMax4824Setup=0x00;

		uchMax4824Setup |= MAX4824_CSB;		// CSbar pin = 1
		uchMax4824Setup |= MAX4824_RESB;	// RESETbar pin = 1
		uchMax4824Setup &= ~MAX4824_SETB;   // SETbar pin = 0
		if(nPsaveActive)
			uchMax4824Setup |= MAX4824_PSAVE;  // if PSAVE active, set PSAVE =1 

		XGpio_SetDataDirection(port, 1, 0x00000000);	// Set the GPIO1 to all be outputs
		XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
}

void max_MAX4824_set_all_inactive(XGpio *port, int nPsaveActive)
/**
* \brief       Set all MAX4824 ports inactive (off).  
* \par         Details
*              All MAX4824 functions are controlled by an 8-bit parallel GPIO port driving its control
*              pins.  This function sets all outputs inactive by driving the RESETbar pin low.  It also
*              allows for PSAVE mode to be enabled/disabled.
*
* \param[in]   *port                 - pointer to the GPIO port instance
* \param[in]   nPsaveActive		     - True = PSAVE mode on, False = PSAVE mode off
*
* \retval      None
*/
{
		u8 uchMax4824Setup=0x00;

		uchMax4824Setup |= MAX4824_CSB;
		uchMax4824Setup &= ~MAX4824_RESB;
		uchMax4824Setup |= MAX4824_SETB;
		if(nPsaveActive)
			uchMax4824Setup |= MAX4824_PSAVE;

		XGpio_SetDataDirection(port, 1, 0x00000000);	// Set the GPIO1 to all be outputs
		XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
}

void max_MAX4824_set_output_active(XGpio *port,u8 relayAddress, int nPsaveActive)
/**
* \brief       Set one MAX4824 port active (on).  
* \par         Details
*              All MAX4824 functions are controlled by an 8-bit parallel GPIO port driving its control
*              pins.  This function sets one of the outputs active by setting the address lines 
*              appropriately, setting the LVL pin high and toggling CSbar.  It also allows for PSAVE
*              mode to be enabled/disabled at the same time.
*
* \param[in]   *port                 - pointer to the GPIO port instance
* \param[in]   relayAddress          - address of relay to turn on (3 bits)
* \param[in]   nPsaveActive          - True = PSAVE mode on, False = PSAVE mode off
*
* \retval      None
*/
{
	u8 uchMax4824Setup=0x00;

	uchMax4824Setup |= MAX4824_CSB;
	uchMax4824Setup |= MAX4824_RESB;
	uchMax4824Setup |= MAX4824_SETB;

	if(nPsaveActive)
		uchMax4824Setup |= MAX4824_PSAVE;

	XGpio_SetDataDirection(port, 1, 0x00000000);	// Set the GPIO1 to all be outputs
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);

	// Set the SETB bit high
	uchMax4824Setup |= MAX4824_SETB;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(1000);

	// set the address bits
	switch(relayAddress)
	{
		case 0x00:
			uchMax4824Setup = uchMax4824Setup;
			break;
		case 0x01:
			uchMax4824Setup |= MAX4824_A0;
			break;
		case 0x02:
			uchMax4824Setup |= MAX4824_A1;
			break;
		case 0x03:
			uchMax4824Setup |= (MAX4824_A1 | MAX4824_A0);
			break;
		case 0x04:
			uchMax4824Setup |= MAX4824_A2;
			break;
		case 0x05:
			uchMax4824Setup |= (MAX4824_A2 | MAX4824_A0);
			break;
		case 0x06:
			uchMax4824Setup |= (MAX4824_A2 | MAX4824_A1);
			break;
		case 0x07:
			uchMax4824Setup |= (MAX4824_A2 | MAX4824_A1 | MAX4824_A0);
			break;
	}

	// Set the Level bit high (and write the relay address)
	uchMax4824Setup |= MAX4824_LVL;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(ABOUT_ONE_SECOND/20);

	// Set the CS bit low (active)
	uchMax4824Setup &= ~MAX4824_CSB;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(ABOUT_ONE_SECOND/20);

	// Set the CS bit high (in-active)
	uchMax4824Setup |= MAX4824_CSB;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(ABOUT_ONE_SECOND/20);
}

void max_MAX4824_set_powersave(XGpio *port, int nPsaveActive)
/**
* \brief       Activate the powersave mode on the MAX4824.  
*
* \param[in]   *port                 - pointer to the GPIO port instance
* \param[in]   nPsaveActive          - True = PSAVE mode on, False = PSAVE mode off
*
* \retval      None
*/
{
	u8 uchMax4824Setup=0x00;
	uchMax4824Setup |= MAX4824_CSB;
	uchMax4824Setup |= MAX4824_RESB;
	uchMax4824Setup |= MAX4824_SETB;
	if(nPsaveActive)
		uchMax4824Setup |= MAX4824_PSAVE;

	XGpio_SetDataDirection(port, 1, 0x00000000);	// Set the GPIO1 to all be outputs
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
}

void max_MAX4824_set_output_inactive(XGpio *port, u8 relayAddress, int nPsaveActive)
/**
* \brief       Set one MAX4824 port inactive (off).  
* \par         Details
*              All MAX4824 functions are controlled by an 8-bit parallel GPIO port driving its control
*              pins.  This function sets one of the outputs inactive by setting the address lines 
*              appropriately, setting the LVL pin low and toggling CSbar.  It also allows for PSAVE
*              mode to be enabled/disabled at the same time.
*
* \param[in]   *port                 - pointer to the GPIO port instance
* \param[in]   relayAddress          - address of relay to turn off (3 bits)
* \param[in]   nPsaveActive          - True = PSAVE mode on, False = PSAVE mode off
*
* \retval      None
*/
{
	u8 uchMax4824Setup=0x00;
	uchMax4824Setup |= MAX4824_CSB;
	uchMax4824Setup |= MAX4824_RESB;
	uchMax4824Setup |= MAX4824_SETB;
	if(nPsaveActive)
		uchMax4824Setup |= MAX4824_PSAVE;

	XGpio_SetDataDirection(port, 1, 0x00000000);	// Set the GPIO1 to all be outputs
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);

	// set the address bits
	switch(relayAddress)
	{
		case 0x00:
			uchMax4824Setup = uchMax4824Setup;
			break;
		case 0x01:
			uchMax4824Setup |= MAX4824_A0;
			break;
		case 0x02:
			uchMax4824Setup |= MAX4824_A1;
			break;
		case 0x03:
			uchMax4824Setup |= (MAX4824_A1 | MAX4824_A0);
			break;
		case 0x04:
			uchMax4824Setup |= MAX4824_A2;
			break;
		case 0x05:
			uchMax4824Setup |= (MAX4824_A2 | MAX4824_A0);
			break;
		case 0x06:
			uchMax4824Setup |= (MAX4824_A2 | MAX4824_A1);
			break;
		case 0x07:
			uchMax4824Setup |= (MAX4824_A2 | MAX4824_A1 | MAX4824_A0);
			break;
	}

	// Set the Level bit low (and write the relay address)
	uchMax4824Setup &= ~MAX4824_LVL;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(1000);

	// Set the SETB bit high
	uchMax4824Setup |= MAX4824_SETB;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(1000);


	// Set the CS bit low (active)
	uchMax4824Setup &= ~MAX4824_CSB;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(1000);

	// Set the CS bit high (in-active)
	uchMax4824Setup |= MAX4824_CSB;
	XGpio_DiscreteWrite(port, 1, uchMax4824Setup);
	delay(1000);
}

int max_MAX31723_set_alarm(float fTemp, u32 unPeripheralAddressSPI, u8 uchAlarmType)
/**
* \brief       Set one of the MAX31723 temperature alarms.  
* \par         Details
*              This function sets either the high or low temperature alarms in the MAX31723.  The
*              units for these settings are degrees centigrade.
*
* \param[in]   fTemp                    - set point temperature in degrees C
* \param[in]   unPeripheralAddressSPI   - address of the SPI peripheral in Microblaze memory map
* \param[in]   uchAlarmType             - True = PSAVE mode on, False = PSAVE mode off
*
* \retval      Always True
*/
{
	u8 auchOutputBuffer[3];
	u8 auchReadBuffer[3];
	int nTemp=0;
	float fTempValue=0.0f;
	int nReturnVal=TRUE;
	int nValueToWrite=0;

	auchOutputBuffer[0]=0;
	auchOutputBuffer[1]=0;
	auchOutputBuffer[2]=0;
	auchReadBuffer[0]=0;
	auchReadBuffer[1]=0;
	auchReadBuffer[2]=0;

	// The upper and lower boundaries for the part are -55.0 to 125.0 degC
	fTempValue = fTemp;
	if(fTempValue >= 125.0f)
		fTempValue = 125.0f;
	else if(fTempValue <= -55.0f)
		fTempValue = -55.0f;

	// The 31723 temperature readings are signed 12 bits long, and each bit represents 1/16th of a degree C
	// bits[11:4] fill the MSB byte
	// bits[3:0] fill the LSB byte upper nibble
	//fTempValue = fTempValue  * 16.0f;
	nValueToWrite = (int)fTempValue;
	nValueToWrite = nValueToWrite << 4;

	// #define MAX31723_LOW_ALARM_WRITE 0x85
	// #define MAX31723_HIGH_ALARM_WRITE 0x83

	if(uchAlarmType==MAX31723_LOW_ALARM_WRITE)
		auchOutputBuffer[0] = MAX31723_LOW_ALARM_WRITE;
	else
		auchOutputBuffer[0] = MAX31723_HIGH_ALARM_WRITE;

	nTemp = (nValueToWrite & 0xFF0);
	nTemp = nTemp >> 4;
	auchOutputBuffer[2] = (u8)nTemp;  // MSB byte is 2nd

	nTemp = (nValueToWrite & 0x000F);			//  mask off the LSB bits
	nTemp = nTemp << 4;  						//  We only have 4 bits remaining of our 12 bit word,
												//  the lower nibble needs to be shifted to the upper nibble
	auchOutputBuffer[1] = (u8)nTemp;

	SpiRW(unPeripheralAddressSPI,1,0,(u8*)&auchOutputBuffer,(u8*)&auchReadBuffer,3,1);  // send

	return(nReturnVal);
}

int max_MAX31723_get_temp(float *fTemp, u32 unPeripheralAddressSPI, u8 uchTemperatureRegister)
/**
* \brief       Retrieve one of the the 12-bit temperature registers from the MAX31723 as a floating point (deg C)  
* \par         Details
*              This function reads any of the three temperature registers in the MAX31723.  These registers are
*              the Temperature, Thigh Alarm and Tlow Alarm.  the register to be read is specified buy the input
*              parameter uchTemperatureRegister which should be set to one of the three constants:
* \n           MAX31723_LOW_ALARM_READ, MAX31723_HIGH_ALARM_READ or MAX31723_TEMP_READ
*
* \param[out]  *fTemp                   - temperature reading is stored at fTemp
* \param[in]   unPeripheralAddressSPI   - address of the SPI peripheral in Microblaze memory map
* \param[in]   uchTemperatureRegister   - The register to be read
*
* \retval      Always True
*/
{
	u8 auchOutputBuffer[3];
	u8 auchReadBuffer[3];
	int nTemp=0;
	float fTempValue=0.0f;
	int nReturnVal=TRUE;

	auchReadBuffer[0]=0;
	auchReadBuffer[1]=0;
	auchReadBuffer[2]=0;

	// Setup the 31723 to continuously make temp readings
	auchOutputBuffer[0] = 0x80; // Configuration/status register
	auchOutputBuffer[1] = 0x06; // Default settings, except use 12 bit resolution (instead of 9) and continuously make temp conversions
	SpiRW(unPeripheralAddressSPI,1,0,(u8*)&auchOutputBuffer,(u8*)&auchReadBuffer,2,1);  // send

	// wait a half a second so that the (now 12-bit) temp sensor readings propagate to the SPI interface
	delay(ABOUT_ONE_SECOND/2);

	// Read the LSB and MSB temperature registers
	if(uchTemperatureRegister == MAX31723_LOW_ALARM_READ)
		auchOutputBuffer[0] = MAX31723_LOW_ALARM_READ; 	// 0x05 = (low) alarm register
	else if(uchTemperatureRegister == MAX31723_HIGH_ALARM_READ)
		auchOutputBuffer[0] = MAX31723_HIGH_ALARM_READ; // 0x03 = (high) alarm register
	else
		auchOutputBuffer[0] = MAX31723_TEMP_READ; 		// 0x01 = temperature register

	auchOutputBuffer[1] = 0x00; // Since this is a read register address, these extra bytes are are ignored by the 31723
	auchOutputBuffer[2] = 0x00; // Since this is a read register address, these extra bytes are are ignored by the 31723
	SpiRW(unPeripheralAddressSPI,1,0,(u8*)&auchOutputBuffer,(u8*)&auchReadBuffer,3,1);  // send

	// convert to approximate floating point
	nTemp = 0;
	nTemp = (int)auchReadBuffer[2];
	nTemp = nTemp << 4;
	nTemp |= (int)((auchReadBuffer[1] & 0xF0)>>4);
	if((nTemp & 0x00000800)==0x00000800) // is the 12bit bit set?  If so, set the other bits for the 2s complement negative value
		nTemp |= 0xFFFFF000;

	fTempValue = (float)nTemp;
	fTempValue = fTempValue / 16.0f;
	*fTemp = fTempValue;
	return(nReturnVal);
}

int max_MAX31855_read_internalTemp(float *fTemp, u32 unPeripheralAddressSPI)
/**
* \brief       Retrieve the temperature reading from the internal temperature sensor in the MAX31855.  
* \par         Details
*              This function reads the INTERNAL temperature data in the MAX31855.  This is the 12-bit temperature
*              measured by the internal temperature sensor which is used in the cold junction compensation.
*
* \param[out]  *fTemp                   - temperature reading is stored at fTemp
* \param[in]   unPeripheralAddressSPI   - address of the SPI peripheral in Microblaze memory map
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchOutputBuffer[4];
	u8 auchReadBuffer[4];
	int nTemporaryValue=0;
	int nTemporaryValue2=0;
	int i;
	int nReturnVal=TRUE;
	float fTempValue=0.0f;

	for(i=0;i<4;i++)
	{
		auchOutputBuffer[i]=0;
		auchReadBuffer[i]=0;
	}
	SpiRW(unPeripheralAddressSPI,0,0,(u8*)&auchOutputBuffer[0],(u8*)&auchReadBuffer[0],4,0);  // Write/Read (3) byte
	//bit_bang_spi_read(port, auchReadBuffer, 4, FALSE);

	// Check for various error codes
	if(((auchReadBuffer[3] & 0x01)==0x01) ||			// Open Circuit
			((auchReadBuffer[3] & 0x02)==0x02) ||		// Short to GND
			((auchReadBuffer[3] & 0x04)==0x04) ||		// Short to VCC
			((auchReadBuffer[1] & 0x01)==0x01))			// Fault
	{
		// Set all data to zero
		nReturnVal=FALSE;
	}
	else
	{
		// Internal Temp
		nTemporaryValue = auchReadBuffer[2];  		// bits 11..4
		nTemporaryValue = nTemporaryValue << 4;
		nTemporaryValue2 = auchReadBuffer[3];
		nTemporaryValue2 = nTemporaryValue2 >> 4;
		nTemporaryValue |= nTemporaryValue2;
		if((auchReadBuffer[2] & 0x80)==0x80)		// Check the sign bit and sign-extend if need be
			nTemporaryValue |= 0xFFFFF800;
		fTempValue = (float)nTemporaryValue;
		fTempValue = fTempValue / 16.0f;
		*fTemp = fTempValue;
		nReturnVal = TRUE;
	}
	return(nReturnVal);
}

int max_MAX31855_read_thermoCoupleTemp(float *fTemp, u32 unPeripheralAddressSPI)
/**
* \brief       Retrieve the temperature reading from the external thermocouple probe connected to the MAX31855.  
* \par         Details
*              This function reads the INTERNAL temperature data in the MAX31855.  This is the 12-bit temperature
*              measured by the internal temperature sensor which is used in the cold junction compensation.
*
* \param[out]  *fTemp                   - temperature reading is stored at fTemp
* \param[in]   unPeripheralAddressSPI   - address of the SPI peripheral in Microblaze memory map
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchOutputBuffer[4];
	u8 auchReadBuffer[4];
	int nTemporaryValue=0;
	int nTemporaryValue2=0;
	int i;
	int nReturnVal=TRUE;
	float fTempValue=0.0f;

	for(i=0;i<4;i++)
	{
		auchOutputBuffer[i]=0;
		auchReadBuffer[i]=0;
	}

	//bit_bang_spi_read(port, auchReadBuffer, 4, FALSE);
	SpiRW(unPeripheralAddressSPI,0,0,(u8*)&auchOutputBuffer[0],(u8*)&auchReadBuffer[0],4,0);  // Write/Read (4) byte

	// Check for various error codes
	if(((auchReadBuffer[3] & 0x01)==0x01) ||			// Open Circuit
			((auchReadBuffer[3] & 0x02)==0x02) ||		// Short to GND
			((auchReadBuffer[3] & 0x04)==0x04) ||		// Short to VCC
			((auchReadBuffer[1] & 0x01)==0x01))			// Fault
	{
		// Set all data to zero
		nReturnVal=FALSE;
	}
	else
	{
		// Internal Temp
		nTemporaryValue = auchReadBuffer[0];  		// bits 11..4
		nTemporaryValue = nTemporaryValue << 6;
		nTemporaryValue2 = auchReadBuffer[1];
		nTemporaryValue2 = nTemporaryValue2 >> 2;
		nTemporaryValue |= nTemporaryValue2;
		if((auchReadBuffer[0] & 0x80)==0x80)		// Check the sign bit and sign-extend if need be
			nTemporaryValue |= 0xFFFFE000;

		fTempValue = (float)nTemporaryValue;
		fTempValue = fTempValue / 4.0f;
		*fTemp = fTempValue;
		nReturnVal = TRUE;
	}
	return(nReturnVal);
}

int max_DS1086L_setup(u32 unPeripheralAddressI2C, XGpio *port)
/**
* \brief       Set up the DS1086 for normal operation.  
* \par         Details
*              The DS1086L operating parameters are set up as follows:
* \n           The GPIO output which drives DS1086L OE pin is set to high to enable the output.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   *port                     - pointer to the GPIO port instance
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	int nByteCount=0;
	u8 uchPortData=0;
	int nReturnVal=TRUE;

	// Set the OE (bit 4) to high
	XGpio_SetDataDirection(port, 1, 0xEF);  // Set all the GPIO to inputs except bit 4, which is an output
	uchPortData = 0x10;
	XGpio_DiscreteWrite(port, 1, uchPortData);  // Set bit 4 (OE = Output Enable) high

	auchTxBuffer[0] = 0x02;  // Pre-scalar register is 0x02
	// Oscillator output is divided by a 4 bit value called PRESCALAR
	// PRESCALAR divides the oscillator output by 2 raised to the PRESCALAR power (ie 2^n)
	// PRESCALAR is the lower 2 bits of register 0x02 and the first two bits of register 0x03
	// The following commands set the SPRD Jitter settings to their minimum settings (000000xx)
	// and the prescalar value to 4'b1000
	auchTxBuffer[1] = 0x02;  // Set Jitter values and turn on 2^C prescalar  
	auchTxBuffer[2] = 0x00;  // Set Jitter values and turn on 2^C prescalar

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
			(u8 *)&auchTxBuffer, 3, XIIC_STOP);

	if(nByteCount!=3)
		nReturnVal=FALSE;

	return(nReturnVal);
}

int max_DS1086_get_dac(u32 unPeripheralAddressI2C, unsigned int *unDacValue)
/**
* \brief       Retrieves the present value of the DAC which drives the DS1086 VCO.  
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *unDacValue               - DAC value from DS1086 is stored at unDacValue
*/
{
	u8 auchTxBuffer[3];
	u8 auchRxBuffer[3];
	int nByteCount=0;
	unsigned int unTemp;
	int nReturnVal=TRUE;

	auchTxBuffer[0] = 0x08;  // 10-bit DAC (for VCO) register is 0x08
	auchTxBuffer[1] = 0x00;  
	auchTxBuffer[2] = 0x00;  

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
			(u8 *)&auchRxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	if(nReturnVal==TRUE)
	{
		unTemp = auchRxBuffer[0];
		unTemp = unTemp << 2;
		unTemp |= ((auchRxBuffer[1] & 0xC0) >> 6);
		*unDacValue = unTemp;
	}

	return(nReturnVal);
}

int max_DS1086_set_dac(u32 unPeripheralAddressI2C, unsigned int unDacValue)
/**
* \brief       Set the DAC which drives the DS1086 VCO.  
* \par         Details
*              The DAC within the DS1086 changes the output frequency by approximately 5kHz per step.
* \n           JS[4:3] = Jitter rate,  JS[2:0] = Jitter percentage*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   unDacValue                - 10-bit value to store to the DAC register
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	int nByteCount=0;
	unsigned int unTemp;
	unsigned int unTempMSB;
	unsigned int unTempLSB;
	int nReturnVal=TRUE;

	unTemp=unDacValue;

	if(unTemp>1023)
		nReturnVal=FALSE;
	else
	{
		// Convert the value to the 10-bit value used in the 1086
		unTempMSB = (unTemp & 0x03FC);
		unTempMSB = unTempMSB >> 2;
		unTempLSB = (unTemp & 0x03);
		unTempLSB = unTempLSB << 6;

		// Write the new value to the DAC register
		auchTxBuffer[0] = 0x08;  // 10-bit DAC (for VCO) register is 0x02
		auchTxBuffer[1] = (u8)unTempMSB;  // Set upper 8 bits of DAC
		auchTxBuffer[2] = (u8)unTempLSB;  // Set lower 2 bits of DAC
		nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
			(u8 *)&auchTxBuffer, 3, XIIC_STOP);
		if(nByteCount!=3)
			nReturnVal=FALSE;
	}

	return(nReturnVal);
}

int max_DS1086_set_prescalar(u32 unPeripheralAddressI2C, unsigned int unPreScalarValue)
/**
* \brief       Set the 4-bit prescaler value in the DS1086.
* \par         Details
*              The DS1086L output is prescaled (divided down) by a value between 2^0 (=1) and 2^8 (=256).  The
*              divide ratio is set by the prescaler register.  This is a 4-bit number, although values greater
*              than 8 are recognized as 8.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   unPreScalarValue          - 4-bit prescaler value to set
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	u8 auchRxBuffer[3];
	int nByteCount=0;
	unsigned int unTemp;
	unsigned int unTempMSB;
	unsigned int unTempLSB;
	int nReturnVal=TRUE;

	// The prescalar 16-bit word carries more data than just the prescalar value (see datasheet)
	// so we must read the prescalar value and then modify the bits to change the P3..0 bits
	auchTxBuffer[0] = 0x02;  // Prescalar register is 0x02
	auchTxBuffer[1] = 0x00;  // NULL
	auchTxBuffer[2] = 0x00;  // NULL

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchRxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	// If the read of the prescalar register was successful, set the new value
	if(nReturnVal==TRUE)
	{
		unTempMSB = auchRxBuffer[0];
		unTempLSB = auchRxBuffer[1];

		// Zero the old bits of the pre-scalar value
		unTempMSB &= 0xFC;
		unTempLSB &= 0x3F;

		// PreScalar Value is capped at 8 (per datasheet)
		unTemp = unPreScalarValue;
		if(unTemp >= 8)
			unTemp=8;

		// Set the new bits of the pre-scalar value
		if((unTemp & 0x08)==0x08)  	// bit 3
			unTempMSB |= 0x02;
		if((unTemp & 0x04)==0x04)
			unTempMSB |= 0x01;
		if((unTemp & 0x02)==0x02)
			unTempLSB |= 0x80;
		if((unTemp & 0x01)==0x01) 	// bit 0
			unTempLSB |= 0x40;

		// Write the bits back to the pre-scalar register
		auchTxBuffer[0] = 0x02;  // Prescalar register is 0x02
		auchTxBuffer[1] = (u8)unTempMSB;
		auchTxBuffer[2] = (u8)unTempLSB;
		nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
			(u8 *)&auchTxBuffer, 3, XIIC_STOP);
		if(nByteCount!=3)
			nReturnVal=FALSE;
	}

	return(nReturnVal);
}

int max_DS1086_get_prescalar(u32 unPeripheralAddressI2C, unsigned int *unPreScalarValue)
/**
* \brief       Retrieve the 4-bit prescaler value in the DS1086.
* \par         Details
*              The DS1086L output is prescaled (divided down) by a value between 2^0 (=1) and 2^8 (=256).  The
*              divide ratio is set by the prescaler register.  This is a 4-bit number, although values greater
*              than 8 are recognized as 8.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *unPreScalarValue         - the 4-bit prescaler value retrived is stored at to set
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	u8 auchRxBuffer[3];
	int nByteCount=0;
	unsigned int unTemp;
	int nReturnVal=TRUE;

	// The prescalar 16-bit word carries more data than just the prescalar value (see datasheet)
	// so we must read the prescalar value and then modify the bits to change the P3..0 bits
	auchTxBuffer[0] = 0x02;  // Prescalar register is 0x02
	auchTxBuffer[1] = 0x00;  // NULL
	auchTxBuffer[2] = 0x00;  // NULL

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchRxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	// If the read was successful, parse the bits and set the value @ *unPrescalarValue
	if(nReturnVal==TRUE)
	{
		unTemp = (auchRxBuffer[0] & 0x03);
		unTemp = unTemp << 2;
		unTemp |= ((auchRxBuffer[1] & 0xC0) >> 6);
		*unPreScalarValue = unTemp;
	}
	return(nReturnVal);
}

int max_DS1086_get_jitter(u32 unPeripheralAddressI2C, unsigned int *unJitterValue)
/**
* \brief       Retrieve the 5-bit jitter register value in the DS1086.
* \par         Details
*              This function returns the 5-bit value for the jitter setting.  This number contains the following bits:
* \n           JS[4:3] = Jitter rate,  JS[2:0] = Jitter percentage
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *unJitterValue            - the 5-bit jitter value retrived is stored to unJitterValue
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	u8 auchRxBuffer[3];
	int nByteCount=0;
	unsigned int unTemp;
	int nReturnVal=TRUE;

	// The prescalar 16-bit word carries more data than just the prescalar value (see datasheet)
	// so we must read the prescalar value and then modify the bits to change the P3..0 bits
	auchTxBuffer[0] = 0x02;  // Prescalar register is 0x02
	auchTxBuffer[1] = 0x00;  // NULL
	auchTxBuffer[2] = 0x00;  // NULL

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchRxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	// If the read was successful, parse the bits and set the value @ *unJitterValue
	if(nReturnVal==TRUE)
	{
		unTemp = (auchRxBuffer[0] & 0xF8);  // five MSBs are JS[4:0]
		unTemp = unTemp >> 3;
		*unJitterValue = unTemp;
	}
	return(nReturnVal);
}

int max_DS1086_set_jitter(u32 unPeripheralAddressI2C, unsigned int unJitterValue)
/**
* \brief       Set the 5-bit jitter register value in the DS1086.
* \par         Details
*              This function sets the 5-bit value for the jitter setting.  This number contains the following bits:
* \n           JS[4:3] = Jitter rate,  JS[2:0] = Jitter percentage
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[in]   unJitterValue             - the 5-bit jitter value to set
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	u8 auchRxBuffer[3];
	int nByteCount=0;
	unsigned int unTempMSB;
	unsigned int unTempLSB;
	int nReturnVal=TRUE;

	// The prescalar 16-bit word carries more data than just the jitter JS[4:0] value (see datasheet)
	// so we must read the prescalar value and then modify the bits to change the JS[4:0] bits
	auchTxBuffer[0] = 0x02;  // Prescalar register is 0x02
	auchTxBuffer[1] = 0x00;  // NULL
	auchTxBuffer[2] = 0x00;  // NULL

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchRxBuffer, 2, XIIC_STOP);
	if(nByteCount!=2)
		nReturnVal=FALSE;

	// If the read of the prescalar register was successful, set the new jitter value
	if(nReturnVal==TRUE)
	{
		unTempMSB = auchRxBuffer[0];
		unTempLSB = auchRxBuffer[1];

		// Zero the old bits of the jitter value (5 MSBs in the MSB byte)
		unTempMSB &= 0x07;

		// Set the new bits of the jitter value
		if((unJitterValue & 0x10)==0x10)  	// bit 4
			unTempMSB |= 0x80;
		if((unJitterValue & 0x08)==0x08)
			unTempMSB |= 0x40;
		if((unJitterValue & 0x04)==0x04)
			unTempMSB |= 0x20;
		if((unJitterValue & 0x02)==0x02)
			unTempMSB |= 0x10;
		if((unJitterValue & 0x01)==0x01) 	// bit 0
			unTempMSB |= 0x08;

		// Write the bits back to the pre-scalar register
		auchTxBuffer[0] = 0x02;  // Prescalar register is 0x02
		auchTxBuffer[1] = (u8)unTempMSB;
		auchTxBuffer[2] = (u8)unTempLSB;
		nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
			(u8 *)&auchTxBuffer, 3, XIIC_STOP);
		if(nByteCount!=3)
			nReturnVal=FALSE;
	}

	return(nReturnVal);
}

int max_DS1086_get_offset_delta(u32 unPeripheralAddressI2C,  int *nOffsetDelta)
/**
* \brief       Retrieve the 5-bit offset/delta register value in the DS1086.
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  *nOffsetDelta             - the 5-bit offset value retrived is stored to nOffsetDelta
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	u8 auchRxBuffer[3];
	int nByteCount=0;
	int nTempRange=0;
	int nTempOffset=0;
	int nTempOffsetDelta=0;
	int nReturnVal=TRUE;

	// Read the Range Register (factory default)
	auchTxBuffer[0] = 0x37;  // Range register is 0x37
	auchTxBuffer[1] = 0x00;  // NULL
	auchTxBuffer[2] = 0x00;  // NULL

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;
	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	// If the read was successful, read the offset register
	if(nReturnVal==TRUE)
	{
		nTempRange = auchRxBuffer[0] & 0x1F; // LSB 5 bits

		// Read the Offset Register
		auchTxBuffer[0] = 0x0E;  // Offset register is 0x0E
		auchTxBuffer[1] = 0x00;  // NULL
		auchTxBuffer[2] = 0x00;  // NULL

		nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
				(u8 *)&auchTxBuffer, 1, XIIC_STOP);
		if(nByteCount!=1)
			nReturnVal=FALSE;
		nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
				(u8 *)&auchRxBuffer, 1, XIIC_STOP);
		if(nByteCount!=1)
			nReturnVal=FALSE;

		// If the offset register read was successful, subtract the two values to get the offset value
		if(nReturnVal==TRUE)
		{
			nTempOffset=auchRxBuffer[0] & 0x1F;
			nTempOffsetDelta = nTempOffset - nTempRange;
			*nOffsetDelta = nTempOffsetDelta;
		}
	}
	return(nReturnVal);
}

// This function sets the value of the offset/delta register in the DS1086
// The offset allows the Master Oscillator Base Frequency to be set in a range from ~61.4MHz to ~122.8MHz
// The offset value needs to be programmed wrt to a factory calibrated RANGE setting (register 0x37)
// So, this function reads the RANGE setting, and then +/- 6 before setting the OFFSET register (0x0E) with the value
int max_DS1086_set_offset_delta(u32 unPeripheralAddressI2C,  int nOffsetDelta)
/**
* \brief       Set the 5-bit offset register value in the DS1086.
* \par         Details
*              The offset allows the Master Oscillator Base Frequency to be set in a range from ~61.4MHz to ~122.8MHz.
*              The offset value needs to be programmed wrt to a factory calibrated RANGE setting (register 0x37).
*              This function reads the RANGE setting, and then +/- 6 before setting the OFFSET register (0x0E) with the value
*
* \param[in]   unPeripheralAddressI2C    - address of the I2C peripheral in Microblaze memory map 
* \param[out]  nOffsetDelta              - the 5-bit offset value to set
*
* \retval      TRUE if operation succeeded
*/
{
	u8 auchTxBuffer[3];
	u8 auchRxBuffer[3];
	int nByteCount=0;
	int nTemp;
	int nTempOffsetDelta;
	int nReturnVal=TRUE;

	// Read the Range Register
	auchTxBuffer[0] = 0x37;  // Prescalar register is 0x02
	auchTxBuffer[1] = 0x00;  // NULL
	auchTxBuffer[2] = 0x00;  // NULL

	nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchTxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	nByteCount = XIic_Recv(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
		(u8 *)&auchRxBuffer, 1, XIIC_STOP);
	if(nByteCount!=1)
		nReturnVal=FALSE;

	// If the read was successful, parse the bits and set the value @ *unPrescalarValue
	if(nReturnVal==TRUE)
	{
		nTemp = (auchRxBuffer[0] & 0x1F);  // mask off the upper 3 bits (they are unused but always read as one)
		nTempOffsetDelta = nOffsetDelta;

		// Cap the offset delta at +/-6
		if(nTempOffsetDelta < -6)
			nTempOffsetDelta=-6;
		else if(nTempOffsetDelta > 6)
			nTempOffsetDelta=6;

		// Calculate the offset Delta value (and then make sure it is a positive number)
		nTemp = nTemp + nTempOffsetDelta;
		if(nTemp<0)
			nTemp=0;

		// Write the offset register
		auchTxBuffer[0] = 0x0E;  // OFFSET register is 0x0E
		auchTxBuffer[1] = (u8)nTemp;

		nByteCount = XIic_Send(unPeripheralAddressI2C, DS1086L_IIC_ADDRESS,
			(u8 *)&auchTxBuffer, 2, XIIC_STOP);
		if(nByteCount!=2)
			nReturnVal=FALSE;
	}
	return(nReturnVal);
}
