/** \file maximPmod.c ******************************************************
 * 
 *             Project: Maxim Plug-in Peripheral Modules
 *            Filename: maximPMOD.c
 *         Description: This module contains the Main application for the
 *                      Digilent Nexys-3 board implementation of example
 *                      programs for the Maxim Plug-in Peripheral Modules
 *
 *    Revision History:
 *\n       7-20-12 Rev 1.4 Nathan Young    Initial release
 * 
 *  --------------------------------------------------------------------
 * 
 *    This code follows the following naming conventions.
 * 
 * 	  char                   chPmodValue
 * 	  char (array)           sPmodString[16]
 * 	  float                  fPmodValue
 * 	  int                    nPmodValue
 * 	  int (array)            anPmodValue[16]
 * 	  u16                    uPmodValue
 * 	  u8                     uchPmodValue
 * 	  u8 (array)             auchPmodBuffer[16]
 * 	  unsigned int           unPmodValue
 * 	  int *                  punPmodValue
 * 
 * ------------------------------------------------------------------------- */
/*
 * Copyright (C) 2012 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products retains all ownership rights.
 *
 ***************************************************************************/

#include <stdio.h>
#include "platform.h"
#include "menu.h"
#include "utilities.h"
#include "maximDeviceSpecificUtilities.h"
#include "maximPMOD.h"

#define MAJOR_REVISION 1
#define MINOR_REVISION 4

// Globals
XGpio g_xGpioSevenSegment;
unsigned int g_unSevenSegmentValue=0;
char g_sInputString[ INPUT_STRING_MAX_SIZE ];				//This string will hold user input from terminal.
XGpio g_xGpioPmodPortA;
XGpio g_xGpioPmodPortB;

XGpio g_gpButtons;
XGpio g_xGpioLed;
u8 g_PmodPortMuxSettings=0;
struct maximProduct s_maximProductArray[16];  // support for up to 16 Maxim ICs

int main()
/**
* \brief       Main() function for Analog Essentials example program.
* \par         Details
*              This function sets up and initializes the FPGA and hardware, displays the root menu via
*              Hyperterminal, then dispatches inidividual demo programs for specific module based on
*              user's keypress selection.
*
* \param       None
*
* \retval      Always TRUE
*/
{
	// Variables for the main() function
	u8 uchInput=0;
	int nMenuState=0;
	int i=0;
	XUartLite Uart;

	// Xilinx provided utility to initialize the hardware/firmware drivers and enable/disable cache
	init_platform();

	// Clear the Hyperterminal screen by sending printf("\033[2J");
	menu_cls();

	// Initialize the USB UART on the NEXYS-3 board (used to communicate with Hyperterminal)
	XUartLite_Initialize(&Uart,DEFAULT_HYPERTERMINAL_UART_ID);
	XUartLite_ResetFifos(&Uart);
	delay(100);

	// Toggle the LEDs so that the user knows the board is awake
	XGpio_Initialize(&g_xGpioLed, XPAR_LEDS_4BITS_DEVICE_ID);
	XGpio_SetDataDirection(&g_xGpioLed, 1, 0x00000000);	// Set the LED peripheral to outputs
	led_knight_rider(&g_xGpioLed,2);

	// Initialize the GPIO connected to PMOD Ports A..D
	XGpio_Initialize(&g_xGpioPmodPortA, XPAR_AXI_GPIO_0_DEVICE_ID);
	XGpio_Initialize(&g_xGpioPmodPortB, XPAR_AXI_GPIO_1_DEVICE_ID);
	XGpio_SetDataDirection(&g_xGpioPmodPortA, 1, 0x00000000);	// Set the GPIO1 to all be outputs
	XGpio_SetDataDirection(&g_xGpioPmodPortB, 1, 0x00000000);	// Set the GPIO1 to all be outputs

	// Load the supported Maxim products & their menu information into an array of structures
	menu_load_supported_maxim_products((struct maximProduct *)&s_maximProductArray[0]);

	// Configure the PMOD Port Multiplexers for the default PMOD interfaces (ports a,b,c,d)
	max_configure_PMOD_port(PMOD_PORT_TYPE_I2C,PMOD_PORT_TYPE_SPI,PMOD_PORT_TYPE_GPIO,PMOD_PORT_TYPE_UART);

	// Clear the Screen, and then display the big Maxim banner for about 2 seconds
	menu_cls();
	menu_print_maxim_banner_big();
	printf("Maxim Analog Essentials for the Avnet LX9 board\r\n");
	printf("Revision v%d.%d\r\n",MAJOR_REVISION,MINOR_REVISION);

	// printf("Hello World\r\n");

	delay(ABOUT_ONE_SECOND * 3);
	//print_seven_segment_number(0.0f);

	/* The primary/main menu loop runs infinitely to display an output similar to the following:
	 *
	 * Press a key/number to test a Peripheral Module:
		{0} DS1086L   Spread Spectrum Econ Oscillator
		{1} DS3231M   I2C Real-Time Clock
		{2} MAX3232   RS-232 Transceiver
		{3} MAX4824   Octal Relay Driver
		{4} MAX5216   SPI-Compatible, High performance 16 bit DAC
		{5} MAX5487   Dual, 256-tap SPI Digital Potentiometer
		{6} MAX5825   I2C Octal DAC
		{7} MAX7304   I2C-Interfaced 16-Port, GPIO and LED Driver
		{8} MAX9611   Current sense amplifier with OpAmp and ADC
		{9} MAX11205  16 bit Delta-Sigma ADC with 2-Wire interface
		{A} MAX14840  RS-485 Transceiver
		{B} MAX31723  DS31723 Digital Thermometer
		{C} MAX31855  Thermocouple to Digital Converter
		{D} MAX44000  I2C-Interfaced Ambient Light and Proximity Sensor
		>>

		The main menu is printed to the screen, and then a blocking call to retrieve a single keypress from the UART is issued.
		The result of that keypress is used to select which menu to display next.
		For example, the MAX44009 is menu item (1).
		In ASCII, (1) is represented by decimal 49
		MENU_MAX is defined with:    #define MENU_MAX44009 49
		and then 'case MENU_MAX:' is used to launch the menu for the MAX44009
		Menu selections of A, B, C, D are also supported.  (lowercase a,b,c,d are aliased to A,B,C,D)
	*/
	while(1)
	{
		switch(nMenuState)
		{
			case MAIN_MENU:
				menu_cls();
				menu_print_main_menu((struct maximProduct *)&s_maximProductArray[0]);
				nMenuState = WAIT_KEYPRESS;
				break;
			case WAIT_KEYPRESS:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				if(uchInput<=9)
					uchInput += 48;  // Add 48 to get the ASCII value
				nMenuState = WAIT_KEYPRESS;
				for(i=0;i<NUMBER_SUPPORTED_PERIPHERAL_MODULES;i++)
				{
					if(uchInput==s_maximProductArray[i].uchKeypress)
					{
						nMenuState = uchInput;
						i=NUMBER_SUPPORTED_PERIPHERAL_MODULES;
					}
				}
				break;

			case MENU_MAX5825:
				menu_cls();
				menu_MAX5825();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX3232:
				menu_cls();
				menu_MAX3232();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX9611:
				menu_cls();
				menu_MAX9611();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX5487:
				menu_cls();
				menu_MAX5487();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX5216:
				menu_cls();
				menu_MAX5216();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX11205:
				menu_cls();
				menu_MAX11205();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX7304:
				menu_cls();
				menu_MAX7304();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX44000:
				menu_cls();
				menu_MAX44000();
				nMenuState = MAIN_MENU;
				break;
			case MENU_DS3231M:
				menu_cls();
				menu_DS3231M();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX4824:
				menu_cls();
				menu_MAX4824();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX14840:
				menu_cls();
				menu_MAX14840();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX31723:
				menu_cls();
				menu_MAX31723();
				nMenuState = MAIN_MENU;
				break;
			case MENU_DS1086L:
				menu_cls();
				menu_DS1086L();
				nMenuState = MAIN_MENU;
				break;
			case MENU_MAX31855:
				menu_cls();
				menu_MAX31855();
				nMenuState = MAIN_MENU;
				break;
			default:
				printf("Invalid Keypress, please try again\r\n");
				delay(ABOUT_ONE_SECOND * 5);
				nMenuState = MAIN_MENU;
				break;
		}
	}

    cleanup_platform();
    return 0;
}
